function [Ex,Ey,Ez,betaTE,betaTM] = E_swg(x, nco, ncl, rho, lambda)

k0    = 2*pi/lambda;
V     = k0*rho*sqrt(nco^2 - ncl^2);
Delta = (nco^2 - ncl^2)/(2*nco^2);

core     = find( abs(x) <= 1);
cladL    = find(     x  < -1);
cladR    = find(     x  >  1);

root_guess = 0.2;

%TE mode
[thetaTE, resTE] = fzero(@EigenValueTE, root_guess);

U = Ufun(thetaTE);
W = Wfun(thetaTE);

betaTE   = BetaOfU(U);
Ey       = exp(-W*abs(x))/exp(-W);
Ey(core) = cos(U*x(core))/cos(U);

display(thetaTE);
display(resTE);
display(U);
display(W);
display(betaTE);

figure(1);
t = linspace(0,pi*0.5,1000);
u = Ufun(t);
w = Wfun(t);
a = u.*tan(u) - w;
%plot(t,a);


%TM mode
[thetaTM, resTM] = fzero(@EigenValueTM, thetaTE);

U = Ufun(thetaTM);
W = Wfun(thetaTM);

betaTM   = BetaOfU(U);
neffTM   = betaTM/k0;

Ex       =  nco^2/ncl^2*exp(-W*abs(x))/exp(-W);
Ex(core) =  cos(U*x(core))/cos(U);

Ez(cladL)= +W/rho/betaTM*nco^2/ncl^2*exp(-W*abs(x(cladL)))/exp(-W);
Ez(cladR)= -W/rho/betaTM*nco^2/ncl^2*exp(-W*abs(x(cladR)))/exp(-W);
Ez(core) = -W/rho/betaTM*nco^2/ncl^2*sin(U*x(core))/sin(U);

display(thetaTM);
display(resTM);
display(U);
display(W);
display(V);
display(betaTM);
display(neffTM);

display(V);

%%%%%% auxiliary functions %%%%%%%%%%%%%%%%%%%%

function b = BetaOfU(u)
 b = 1/rho*sqrt(V^2/(2*Delta) - u^2);
end

function  u = Ufun(t) 
  u = V*sin(t);
end

function  w = Wfun(t) 
  w = V*cos(t);
end

function f = EigenValueTE(t)
  Uaux = Ufun(t);
  Waux = Wfun(t);
  f    = Uaux*tan(Uaux)-Waux;
end

function f = EigenValueTM(t)
  Uaux = Ufun(t);
  Waux = Wfun(t);
  f    = ncl^2*Uaux*tan(Uaux) - nco^2*Waux;
end


end
